/*
 * ==============================================================================
 *  Name        : SimpleCube.cpp
 *  Part of     : OpenGLEx / SimpleCube
 *
 *  Copyright (c) 2004-2006 Nokia Corporation.
 *  This material, including documentation and any related
 *  computer programs, is protected by copyright controlled by
 *  Nokia Corporation.
 * ==============================================================================
 */


// INCLUDE FILES
#include <e32std.h>
#include "SimpleCube.h"

// CONSTANTS

/** Vertice coordinates for the cube. */
static const GLbyte vertices[8 * 3] =
     {
     -1,  1,  1,
      1,  1,  1,
      1, -1,  1,
     -1, -1,  1,

     -1,  1, -1,
      1,  1, -1,
      1, -1, -1,
     -1, -1, -1
     };

/** Colors for vertices (Red, Green, Blue, Alpha). */
static const GLubyte colors[8 * 4] =
    {
    0  ,255,  0,255,
    0  ,  0,255,255,
    0  ,255,  0,255,
    255,  0,  0,255,

    0  ,  0,255,255,
    255,  0,  0,255,
    0  ,  0,255,255,
    0  ,255,  0,255
    };


/**
 * Indices for drawing the triangles.
 * The color of the triangle is determined by
 * the color of the last vertex of the triangle.
 */
static const GLubyte triangles[12 * 3] =
    {
    /* front */
    1,0,3,
    1,3,2,

    /* right */
    2,6,5,
    2,5,1,

    /* back */
    7,4,5,
    7,5,6,

    /* left */
    0,4,7,
    0,7,3,

    /* top */
    5,4,0,
    5,0,1,

    /* bottom */
    3,7,6,
    3,6,2
    };

/** First set of indices for drawing the triangle fans. */
static const GLubyte fanOne[6 * 3] =
    {
    1,0,3,
    1,3,2,
    1,2,6,
    1,6,5,
    1,5,4,
    1,4,0
    };


/** Second set of indices for drawing the triangle fans. */
static const GLubyte fanTwo[6 * 3] =
    {
    7,4,5,
    7,5,6,
    7,6,2,
    7,2,3,
    7,3,0,
    7,0,4
    };



// ============================ MEMBER FUNCTIONS ===============================
CSimpleCube* CSimpleCube::NewL( TUint aWidth, TUint aHeight )
{
	/* Symbian 2-phase constructor. Calls both the default
	   C++ constructor and Symbian ConstructL methods */
	CSimpleCube* self = new (ELeave) CSimpleCube( aWidth, aHeight );
	CleanupStack::PushL( self );
	self->ConstructL();
	CleanupStack::Pop();
	
	return self;
}

CSimpleCube::CSimpleCube( TUint aWidth, TUint aHeight ) : iScreenWidth( aWidth ), iScreenHeight( aHeight )
{
}

CSimpleCube::~CSimpleCube()
{
}

void CSimpleCube::ConstructL( void )
{
}

void CSimpleCube::AppInit( void )
{
	// Initialize viewport and projection.
	SetScreenSize( iScreenWidth, iScreenHeight );
	
	glClearColor( 0.5f, 0.5f, 0.5f, 1.f );
	glEnable( GL_CULL_FACE  );

	glShadeModel( GL_FLAT );
	glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_FASTEST );

	glEnableClientState( GL_VERTEX_ARRAY );
	glVertexPointer( 3, GL_BYTE, 0, vertices );
	glEnableClientState( GL_COLOR_ARRAY );
	glColorPointer( 4, GL_UNSIGNED_BYTE, 0, colors );
	
	// Set the initial drawing mode
    iDrawingMode = ETriangles;
}

void CSimpleCube::AppExit( void )
{
}

void CSimpleCube::DrawBox( GLfloat aSizeX, GLfloat aSizeY, GLfloat aSizeZ )
{
	glScalef( aSizeX, aSizeY, aSizeZ );
	
	if ( iDrawingMode == ETriangles )
	{
	    glDrawElements( GL_TRIANGLES, 12 * 3, GL_UNSIGNED_BYTE, triangles );
	}
	else if ( iDrawingMode == ETriangleFans )
	{
	    glDrawElements( GL_TRIANGLE_FAN, 6 * 3, GL_UNSIGNED_BYTE, fanOne );
	    glDrawElements( GL_TRIANGLE_FAN, 6 * 3, GL_UNSIGNED_BYTE, fanTwo );
	}
}

void CSimpleCube::AppCycle( TInt aFrame )
{
	const GLint cameraDistance = 100;
	
	glClear( GL_COLOR_BUFFER_BIT );
	
	/* Animate and draw box */
	glLoadIdentity();
	glTranslatex( 0 , 0 , -cameraDistance << 16 );
	glRotatex( aFrame << 16, 1 << 16,    0   ,    0    );
	glRotatex( aFrame << 15,    0   , 1 << 16,    0    );
	glRotatex( aFrame << 14,    0   ,    0   , 1 << 16 );
	DrawBox( 15.f, 15.f, 15.f );
}
void CSimpleCube::FlatShading( void )
{
	glShadeModel( GL_FLAT );
}

void CSimpleCube::SmoothShading( void )
{
	glShadeModel( GL_SMOOTH );
}

void CSimpleCube::TriangleMode( void )
{
	iDrawingMode = ETriangles;
}

void CSimpleCube::TriangleFanMode( void )
{
	iDrawingMode = ETriangleFans;
}
void CSimpleCube::WhiteBG( void )
{
	glClearColor( 1.0f, 1.0f, 1.0f, 1.f );
}
void CSimpleCube::BlackBG( void )
{
	glClearColor( 0.0f, 0.0f, 0.0f, 1.f );
}

void CSimpleCube::SetScreenSize( TUint aWidth, TUint aHeight )
{
	iScreenWidth  = aWidth;
	iScreenHeight = aHeight;
	
	// Reinitialize viewport and projection.
	glViewport( 0, 0, iScreenWidth, iScreenHeight );
	
	// Recalculate the view frustrum
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	
	GLfloat aspectRatio = (GLfloat)(iScreenWidth) / (GLfloat)(iScreenHeight);
	
	glFrustumf( FRUSTUM_LEFT * aspectRatio, FRUSTUM_RIGHT * aspectRatio,
	            FRUSTUM_BOTTOM, FRUSTUM_TOP,
	            FRUSTUM_NEAR, FRUSTUM_FAR );
	glMatrixMode( GL_MODELVIEW );
}

// End of File

